"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lambda_api_1 = __importDefault(require("lambda-api"));
const inversify_1 = require("inversify");
const Endpoint_1 = require("./Endpoint");
const MiddlewareRegistry_1 = require("./MiddlewareRegistry");
const AppConfig_1 = require("../model/AppConfig");
const ApiRequest_1 = require("../model/ApiRequest");
const ControllerLoader_1 = require("./reflection/ControllerLoader");
const DecoratorRegistry_1 = require("./reflection/DecoratorRegistry");
const timed_1 = require("../util/timed");
/**
 * Server that discovers routes using decorators on controller
 * classes and methods. Processing of requests is preformed by the
 * `lambda-api` package.
 */
let Server = class Server {
    /**
     * Create a new server.
     *
     * @param appConfig Application config to pass to `lambda-api`.
     */
    constructor(appConfig) {
        this.api = lambda_api_1.default(appConfig);
        this._middlewareRegistry = new MiddlewareRegistry_1.MiddlewareRegistry();
    }
    get middlewareRegistry() {
        return this._middlewareRegistry;
    }
    /**
     * Configure the `API` instance from the `lambda-api`
     * package.
     *
     * @param handler Function that takes an `API` instance as a parameter.
     */
    configure(handler) {
        handler(this.api);
    }
    /**
     * Scans the specified path for javascript files and loads these into
     * the current runtime. Importing the files will invoke the decorators
     * declared within them. Note: this scans only the top level files.
     *
     * API decorators register controllers, endpoints, configuration and middleware.
     * A series of endpoints are built using the decorator components and registered
     * with the `lambda-api` package routing engine.
     *
     * Controllers and error interceptors registered by decorators are built using
     * an IOC container, which allows dependency injection.
     *
     * This method must be called before invoking the `processEvent` method.
     *
     * @param controllersPath Path to the directory containing controller `js` files.
     * @param appContainer `InversifyJS` IOC `Container` instance which can build controllers and error interceptors.
     */
    async discoverAndBuildRoutes(controllersPath, appContainer) {
        await ControllerLoader_1.ControllerLoader.loadControllers(controllersPath);
        for (let endpointKey in DecoratorRegistry_1.DecoratorRegistry.Endpoints) {
            let apiEndpoint = new Endpoint_1.Endpoint(DecoratorRegistry_1.DecoratorRegistry.Endpoints[endpointKey], c => appContainer.get(c), ei => appContainer.get(ei), this._middlewareRegistry);
            apiEndpoint.register(this.api);
        }
    }
    /**
     * Takes an API request passed in by AWS Lambda and processes
     * it using the `lambda-api` package.
     *
     * @param request API Gateway or ALB request.
     * @param context Request context.
     * @returns The response.
     */
    async processEvent(request, context) {
        let event = request;
        return await this.api.run(event, context);
    }
};
__decorate([
    timed_1.timed,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, inversify_1.Container]),
    __metadata("design:returntype", Promise)
], Server.prototype, "discoverAndBuildRoutes", null);
__decorate([
    timed_1.timed,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [ApiRequest_1.ApiRequest, Object]),
    __metadata("design:returntype", Promise)
], Server.prototype, "processEvent", null);
Server = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(AppConfig_1.AppConfig)),
    __metadata("design:paramtypes", [AppConfig_1.AppConfig])
], Server);
exports.Server = Server;
//# sourceMappingURL=Server.js.map